import unittest
from InCli import InCli
from InCli.SFAPI import file,file_csv


class Test_Splunk(unittest.TestCase):
    def test_merge_files(self):
        J = file_csv.read('/Users/uormaechea/Downloads/Jw.csv',separator=',')
        apars = file_csv.read('/Users/uormaechea/Downloads/aparsw.csv',separator=',')

        lenJ = len(J)
        num = 0
        for ra in apars:
            found = False
            for i,rj in enumerate(J):
                if rj['parentRequestId'] == ra['parentRequestId']:
                    ra['bandwidth'] = rj['bandwidth']
                    J.pop(i)
                    found = True
                   # print('',sep='.')
                    num = num +1
                    break
            if found: continue
           # else: print(f"Not found: {ra['parentRequestId']}")
        print(f"J {len(J)} {lenJ} apars {len(apars)}  found {num}")
        file_csv.write('/Users/uormaechea/Downloads/output_week.csv',apars)
        print()


#(index=CS189) CASE(00D7a0000005DzR) sourcetype=CASE(applog*:J) | stats count by timestamp,bandwidth,runTime,logName,parentRequestId


from . import Sobjects,tooling,utils
#def get_traceFlag_for_user(userF):
#    userId = Sobjects.IdF('User',userF)
#    q = f"select id, TracedEntityId,logtype, startdate, expirationdate, debuglevelid, debuglevel.apexcode, debuglevel.visualforce from TraceFlag where TracedEntityId = '{userId}'"
#    call = query(q)
#    utils.printFormated(call['records'],fieldsString="Id StartDate ExpirationDate DebugLevel.ApexCode",rename="DebugLevel.ApexCode%ApexCode",separator=' ')

def create_debug_level_incli():
    data = {
        "DeveloperName": "InCli",
        "Language": "pt_BR",
        "MasterLabel": "InCli",
        "Workflow": "FINE",
        "Validation": "INFO",
        "Callout": "INFO",
        "ApexCode": "FINEST",
        "ApexProfiling": "INFO",
        "Visualforce": "FINER",
        "System": "FINE",
        "Database": "FINEST",
        "Wave": "NONE",
        "Nba": "NONE"
    }
    call = tooling.post('DebugLevel',data=data)
    return call

def create_trace_flag_incli_f(userF):
    userId = Sobjects.IdF('User',userF)
    DebugLevelId = tooling.IdF('DebugLevel',"DeveloperName:InCli")
    return create_trace_flag_incli(userId,DebugLevelId)

def create_trace_flag_incli(userId,DebugLevelId):
    data = {
        "TracedEntityId":userId,
        "LogType":"USER_DEBUG",
        "DebugLevelId":DebugLevelId,
        "StartDate":utils.datetime_now_string(),
        "ExpirationDate":utils.datetime_now_string(addMinutes=10)
    }
    call = tooling.post('TraceFlag',data=data)
    return get_trace_flags(call['id'])

    #'7tf3O000001LbmoQAC'

def update_trace_flag_incli(id,minutes=5,start=-2):
    data = {
        "StartDate":utils.datetime_now_string(addMinutes=start),
        "ExpirationDate":utils.datetime_now_string(addMinutes=minutes)
    }
    call = tooling.patch(sobject='TraceFlag',id=id , data=data)
    return get_trace_flags(id)

def delete_trace_Flag(id):
    tooling.delete(sobject='TraceFlag',id=id)

def get_trace_flags(id):
    call = tooling.get(sobject='TraceFlag',id=id)
    return call

def get_InCli_traceflags_for_user(userF):
    try:
        call = get_traceflags_for_user(userF,developerName='InCli')
    except Exception as e:
        if 'invalid ID field' in e.args[0]['error']:
            utils.raiseException("INVALID_USER",f"User {userF} does not exist.")
        raise e
    return call
    
def get_traceflags_for_user(userF,developerName=None):
    userId = Sobjects.IdF('User',userF)

    q = f"select Id,StartDate,ExpirationDate,DebugLevelId,DebugLevel.DeveloperName,ApexCode,ApexProfiling,Callout,Database,LogType,System,TracedEntityId,Validation,Visualforce,Workflow from TraceFlag where TracedEntityId='{userId}'"

    call = tooling.query(q)

    if developerName!=None:
        call2 = [r for r in call['records'] if r['DebugLevel']['DeveloperName'] == developerName]
        if len(call2) > 1:
            print(f"There is more than one traceflag for user {userF} for InCli.")

        if len(call2) > 0:
            return call2[0]
        return None
    return call

def set_incli_traceFlag_for_user(userF):
    InCli_trace_flags = get_InCli_traceflags_for_user(userF)

    if InCli_trace_flags == None:
        InCli_trace_flags = create_trace_flag_incli_f(userF)

    InCli_trace_flags = update_trace_flag_incli(InCli_trace_flags['Id'],5)

    return InCli_trace_flags

def set_incli_traceFlag_for_user_old(userF):
    userId = Sobjects.IdF('User',userF)
    DebugLevelId = tooling.IdF('DebugLevel',"DeveloperName:InCli")   
    if DebugLevelId == None:
        call = create_debug_level_incli()
        DebugLevelId = call['id']  

    q = f"select Id from TraceFlag where DebugLevelId='{DebugLevelId}' and TracedEntityId = '{userId}'"
    call = tooling.query(q)

    if len(call['records']) == 0:
        call = create_trace_flag_incli(userId,DebugLevelId)
        traceFlagId = call['id']
    else:
       traceFlagId = call['records'][0]['Id']

    update_trace_flag_incli(traceFlagId,5)

    return traceFlagId
#	https://appsmobileqms.nos.pt
#   https://appsmobileqms.nos.pt/
#   https://appsmobileqms.nos.pt


import unittest
from InCli import InCli
from InCli.SFAPI import account,restClient,digitalCommerce,digitalCommerceUtil,utils,CPQ,timeStats

class Test_DC(unittest.TestCase):
    def test_catalogs(self):
        restClient.init('DEVNOSCAT4')

        catalogs = digitalCommerce.getCatalogs()

        print()

    def test_getOffers(self):
        restClient.init('DEVNOSCAT4')

    #    catalogs = digitalCommerce.getCatalogs()
        catalogs =['DCTEST','MPOTEST']
        for catalog in catalogs:
            try:
                offers = digitalCommerce.getOfferByCatalogue(catalog)
                print(f"offers: {len(offers)}")
            except Exception as e:
                print(f"{catalog}  {e.args[0]['error']}")

        print()

    def test_create_Basket_config(self):
        restClient.init('DEVNOSCAT4')

        catalog ='DCTEST'
        offer ='C_WOO_MOBILE'
        try:
            details = digitalCommerce.getOfferDetails(catalog,offer)
            digitalCommerce.createBasketAfterConfig(catalog,details)
            print(f"offers: {len(details)}")
        except Exception as e:
            print(f"{catalog}  {e.args[0]['error']}")

        print()

    def test_getOffer_details(self):
        restClient.init('DEVNOSCAT4')

        catalog ='DCTEST'
        offer ='C_WOO_MOBILE'
        try:
            offers = digitalCommerce.getOfferDetails(catalog,offer)
            print(f"offers: {len(offers)}")
        except Exception as e:
            print(f"{catalog}  {e.args[0]['error']}")

        print()

    def test_create_Basket(self):
        restClient.init('DEVNOSCAT4')

        catalog ='DCTEST'
        offer ='C_WOO_MOBILE'
        try:
            basket = digitalCommerce.createBasket(catalog,offer)

            print(f"offers: {len(basket)}")
        except Exception as e:
            print(f"{catalog}  {e.args[0]['error']}")

        print()

    def test_create_Basket_cart(self):
        restClient.init('DEVNOSCAT4')

        catalog ='DCTEST'
        offer ='C_WOO_MOBILE'
        try:
            basket = digitalCommerce.createBasket(catalog,offer)
            accountId ='0013O000017xZ2UQAU'
            digitalCommerce.createCart(accountId,catalog,cartContextKey=basket['cartContextKey'])

            print(f"offers: {len(basket)}")
        except Exception as e:
            print(f"{catalog}  {e.args[0]['error']}")

        print()

        #getItemAttributes {ATT_MOBILE_NUMBER=null, ATT_ORDER_TYPE=null, ATT_PORT_ACTIVE_ICCID=null, ATT_PORT_ACTIVE_MOBILE_TARIFF=null, ATT_PORT_CVP_NUMBER=null, ATT_PORT_DATE=null, ATT_PORT_DOC_UPLOAD_DATE=null, ATT_PORT_MOBILE_NUMBER=null, ATT_PORT_ORIGIN_ISP=null, ATT_PORT_ORIGIN_OWNER_NAME=null, ...}
    
        
    def test_DEVNOSCAT4_C_WOO_MOBILE(self):
        restClient.init('DEVNOSCAT4')
        self.test_WOO_Mobile('DCTEST',userValue=21,iterations=1)

    def test_NOSDEV_C_WOO_MOBILE(self):
        restClient.init('NOSDEV')
        self.test_WOO_Mobile('DC_CAT_WOO_MOBILE')

    def test_WOO_Mobile(self,catalog,userValue=200,iterations=3):
        offer ='C_WOO_MOBILE'

        for i in range(0,iterations):
            try:
                restClient.new_time_record()

                details = digitalCommerce.getOfferDetails(catalog,offer,ts_name='getOfferDetails')

                updated = digitalCommerceUtil.updateOfferField(details,'ATT_NOS_OTT_SUBSCRIPTION_ID','userValues',userValue+i,'code')

                basket = digitalCommerce.createBasketAfterConfig(catalog,updated,ts_name='createBasketAfterConfig')
              #  basket = digitalCommerce.createBasket(catalog,basketAction='AddWithNoConfig',offer=offer)

                bd = digitalCommerce.getBasketDetails(catalog,basket['cartContextKey'])

                filename = restClient.callSave('getBasketDetails_111')

                res = digitalCommerce.addChildBasket(bd,"C_VOICE_MOBILE_TARIFFS:C_VOICE_MOBILE_SERVICE_001",ts_name='add_SERVICE_001')

                res = digitalCommerce.addChildBasket(bd,"C_SIM_E-SIM_CARD:C_SIM_CARD",ts_name='add_C_CIM_CARD')

                res = digitalCommerce.addChildBasket(bd,"C_COMPLEMENT_SERVICE:C_EXTRAS_DATA_3GB",ts_name='add_DATA_3GB')

                accountId ='0013O000017xZ2UQAU'
                res = digitalCommerce.createCart(accountId,catalog,cartContextKey=basket['cartContextKey'],ts_name='createCart')

                orderId = res['orderId']

                res1 = CPQ.deleteCart(orderId,ts_name='deleteCart')


            except Exception as e:
                filename = restClient.callSave('request_DC_111',logRequest=True,logReply=False)
                utils.printException(e)
                #ts.time_no("Error",e.args[0]['errorCode'])
               # ts.print()

        restClient.time_print()

        print()

    def test_DCTEST_C_MPO_Mobile100(self):
        restClient.init('DEVNOSCAT4')

        catalog ='MPOTEST'
        offer ='C_NOS_OFFER_001'
        ts = timeStats.TimeStats()
        ts.new()

        try:
            details = digitalCommerce.getOfferDetails(catalog,offer)

            ts.time('getOfferDetails')

            filename = restClient.callSave('mpodetails111')

            updated = digitalCommerceUtil.updateOfferField(details,'ATT_SERIAL_NUMBER','userValues',1113,'code')

            basket = digitalCommerce.createBasketAfterConfig(catalog,details)
            #basket = digitalCommerce.createBasket(catalog,basketAction='AddWithNoConfig',offer=offer)



            print(f"createBasketAfterConfig: {restClient.getLastCallAllTimes()}")

            bd = digitalCommerce.getBasketDetails(catalog,basket['cartContextKey'])
            ts.time('getBasketDetails')

            filename = restClient.callSave('getBasketDetails_111')

            res = digitalCommerce.addChildBasket(bd,"C_NOS_AGG_EQUIPS_TV_OPT_UMA:C_NOS_EQUIP_TV_005")
            ts.time('add_TV_005')

            res = digitalCommerce.addChildBasket(bd,"C_NOS_AGG_MOBILE_DATA:C_NOS_SERVICE_VM_004")
            ts.time('add_VM_004')

            res = digitalCommerce.addChildBasket(bd,"C_NOS_OFFER_001:C_NOS_AGG_SERVICES_OPT_VM")
            ts.time('add_OPT_VM')

            accountId ='0013O000017xZ2UQAU'
            res = digitalCommerce.createCart(accountId,catalog,cartContextKey=basket['cartContextKey'])
            ts.time('createCart')

            orderId = res['orderId']

            res1 = CPQ.deleteCart(orderId)
            ts.time('deleteCart')

            ts.print()
            print()

        except Exception as e:
            filename = restClient.callSave('request_DC_111',logRequest=True,logReply=False)
            utils.printException(e)
            ts.time_no("Error",e.args[0]['errorCode'])
            ts.print()


        print()

    def test_DCTEST_MOBILE_TRIAL(self):
        restClient.init('DEVNOSCAT4')

        catalog ='DCTEST'
        offer ='PROMO_WOO_MOBILE_TRIAL'
        try:
            details = digitalCommerce.getOfferDetails(catalog,offer)
            print(f"getOfferDetails: {restClient.getLastCallAllTimes()}")

            updated = digitalCommerceUtil.updateOfferField(details,'ATT_NOS_OTT_SUBSCRIPTION_ID','userValues',1113,'code')

           # basket = digitalCommerce.createBasketAfterConfig(catalog,updated)AddAfterConfig
        #    basket = digitalCommerce.createBasket(catalog,basketAction='AddWithNoConfig',offer=offer)
            basket = digitalCommerce.createBasket(catalog,basketAction='AddAfterConfig',offer=offer)

            #basket = digitalCommerce.createBasket(catalog,'',basketAction='AddAfterConfig',productConfig=updated)

            print(f"createBasketAfterConfig: {restClient.getLastCallAllTimes()}")

            bd = digitalCommerce.getBasketDetails(catalog,basket['cartContextKey'])
            print(f"getBasketDetails: {restClient.getLastCallAllTimes()}")

            filename = restClient.callSave('getBasketDetails_111')

            res = digitalCommerce.addChildBasket(bd,"C_VOICE_MOBILE_TARIFFS:C_VOICE_MOBILE_SERVICE_001")
            print(f"addChildBasket: {restClient.getLastCallAllTimes()}")

            res = digitalCommerce.addChildBasket(bd,"C_SIM_E-SIM_CARD:C_SIM_CARD")
            print(f"addChildBasket: {restClient.getLastCallAllTimes()}")

            res = digitalCommerce.addChildBasket(bd,"C_COMPLEMENT_SERVICE:C_EXTRAS_DATA_3GB")
            print(f"addChildBasket: {restClient.getLastCallAllTimes()}")

            accountId ='0013O000017xZ2UQAU'
            res = digitalCommerce.createCart(accountId,catalog,cartContextKey=basket['cartContextKey'])
            print(f"createCart: {restClient.getLastCallAllTimes()}")

            orderId = res['orderId']

            res1 = CPQ.deleteCart(orderId)
            print(f"deleteCart: {restClient.getLastCallAllTimes()}")

            print()

        except Exception as e:
            filename = restClient.callSave('request_DC_111',logRequest=True,logReply=False)
            utils.printException(e)

        print()


from . import query,utils
import simplejson,sys 

def get_errors(where=None,limit=None):
    l = f" limit {limit} " if limit!= None else " limit 50 "
    q = f"""select Id,
                    OwnerId,
                    IsDeleted,
                    Name,
                    CreatedDate,
                    CreatedById,
                    LastModifiedDate,
                    SystemModstamp,
                    vlocity_cmt__Action__c,
                    vlocity_cmt__ErrorTime__c,
                    vlocity_cmt__InputData__c,
                    vlocity_cmt__ObjectName__c,
                    vlocity_cmt__SourceName__c,
                    vlocity_cmt__SourceType__c 
                    from vlocity_cmt__VlocityErrorLogEntry__c order by vlocity_cmt__ErrorTime__c desc  {l}"""

    return query.query(q)

def print_error_list(limit=None):
    res = get_errors(limit=limit)
    if len(res['records'])==0: return print(f"There are no records to pring.")

    onwerIds = [r['OwnerId'] for r in res['records']]
    q = "select Name,Id from User where Id in ($$$IN$$$)"
    res2 = query.query(q,in_list=onwerIds)

    for record in res['records']:
        record['ErrorMessage__c'] = _get_record_error_message(record)
        record['vlocity_cmt__ErrorTime__c'] = record['vlocity_cmt__ErrorTime__c'][0:19]
        record['Owner'] = [r['Name'] for r in res2['records'] if r['Id'] == record['OwnerId']][0]

    print(f"Printing {len(res['records'])} records from vlocity_cmt__VlocityErrorLogEntry__c.")
    utils.printFormated(res['records'],"Id:Owner:vlocity_cmt__SourceType__c:vlocity_cmt__SourceName__c:vlocity_cmt__ErrorTime__c:vlocity_cmt__Action__c:ErrorMessage__c",rename="vlocity_cmt__SourceType__c%sourceType:vlocity_cmt__ErrorTime__c%ErrorTime")

def _get_record_error_message(record):
    input_data = simplejson.loads(record['vlocity_cmt__InputData__c'])

    error_message = 'Other'

    if record['vlocity_cmt__SourceType__c'] == 'Odin':
        error_message = input_data['ErrorMessage__c'][0:70]

    elif record['vlocity_cmt__SourceType__c'] == 'Integration Procedure':
        if  'ErrorMessage'  in input_data:
            if 'ErrorCode' in input_data:
                error_message  = f"{input_data['ErrorCode']} {input_data['ErrorMessage']} "
            else:
                error_message  = f"{input_data['ErrorMessage']} "

        elif 'GenericLogMessage' in input_data:
            error_message = input_data['GenericLogMessage'][0:70]
        else: 
            if 'StepResult' in input_data:
                if 'result' in input_data['StepResult']:
                    if 'Error' in input_data['StepResult']['result']:
                        if 'errorCode' in input_data['StepResult']['result']['Error']:
                            error = input_data['StepResult']['result']['Error']
                            return f"ERROR: {error['errorMessage']} - {error['errorCode']} "
                    if 'ActivisErr' in input_data['StepResult']['result']:
                        if input_data['StepResult']['result']['ActivisErr'] != None:
                            try:
                                return f"HttpCode {input_data['StepResult']['result']['ActivisErr']['eDescription']} "
                            except Exception as e:
                                print(e)
                        else:
                            if 'Verbose' in input_data['StepResult']['result']:
                                if 'System' in input_data['StepResult']['result']['Verbose']:
                                    if 'Tibco' in input_data['StepResult']['result']['Verbose']['System']:
                                        if 'eCodes' in input_data['StepResult']['result']['Verbose']['System']['Tibco']:
                                            eCodes = input_data['StepResult']['result']['Verbose']['System']['Tibco']['eCodes']
                                            return f"TIBCO: {eCodes['eDescription']} - {eCodes['eCode']} "
                    if 'HTTPStatusCode' in input_data['StepResult']['result']:
                        error_message  = f"HttpCode {input_data['StepResult']['result']['HTTPStatusCode']} "
                    if 'errors' in input_data['StepResult']['result']:
                        error_message  = str(input_data['StepResult']['result']['errors'])
                    if 'message' in input_data['StepResult']['result']:
                        return input_data['StepResult']['result']['message']
                    if 'eNative' in input_data['StepResult']['result']:
                        if 'eDescription' in input_data['StepResult']['result']['eNative']:
                            return input_data['StepResult']['result']['eNative']['eDescription']
                        else:
                            if type(input_data['StepResult']['result']['eNative']) is list:
                                enative = input_data['StepResult']['result']['eNative'][0]
                                return enative['eDescription']
                    
                if 'info' in input_data['StepResult']:
                    if 'status' in input_data['StepResult']['info']:
                        error_message  = f"{input_data['StepResult']['info']['status']} {input_data['StepResult']['info']['statusCode']}"
                if 'error' in input_data['StepResult']:
                        error_message  = f"{input_data['StepResult']['error']}"
    return error_message

def print_error_records(last=None):
    limit = last if last!=None else 50
    res = get_errors(limit=limit)

    for record in res['records']:
        record['vlocity_cmt__ErrorTime__c'] = record['vlocity_cmt__ErrorTime__c'][0:19]
        record['ErrorMessage__c'] = _get_record_error_message(record)

        _print_error_record(record,print_renames=False)
        print()
        print()

   # utils.printFormated(res['records'],"Id:vlocity_cmt__ErrorTime__c:vlocity_cmt__SourceType__c:vlocity_cmt__SourceName__c:vlocity_cmt__Action__c:ErrorMessage__c",rename="vlocity_cmt__SourceType__c%sourceType:vlocity_cmt__ObjectName__c%ObjectName:vlocity_cmt__ErrorTime__c%time")

def query_and_print_error_record(id,tofile=False):
    q = f"select fields(all) from vlocity_cmt__VlocityErrorLogEntry__c where Id='{id}'"
    records = query.queryRecords(q)

    if len(records)>0:
        record = records[0]
        _print_error_record(record,tofile=tofile)
    else:
        print(f"Record Id {id} not found.")


def _print_error_record(record,print_renames=True,tofile=False):
    record['ErrorMessage__c'] = _get_record_error_message(record)
    record['vlocity_cmt__ErrorTime__c'] = record['vlocity_cmt__ErrorTime__c'][0:19]


    utils.printFormated(record,"Id:vlocity_cmt__ErrorTime__c:vlocity_cmt__SourceType__c:vlocity_cmt__SourceName__c:vlocity_cmt__Action__c:ErrorMessage__c",rename="vlocity_cmt__SourceType__c%sourceType:vlocity_cmt__ObjectName__c%ObjectName:vlocity_cmt__ErrorTime__c%time",print_renames=print_renames)

    filename = record['Id'] if tofile == True else None

    input_data = simplejson.loads(record['vlocity_cmt__InputData__c'])
  #  if 'Request' in input_data:
  #      input_data['Request'] = simplejson.loads(input_data['Request'])

    utils.print_json(input_data,filename=filename)

    return
    input_data = simplejson.loads(record['vlocity_cmt__InputData__c'])
    json_formatted_str = simplejson.dumps(input_data, indent=2, ensure_ascii=False)


def _extract_error(record):
    inputdata_str = record['vlocity_cmt__InputData__c']
    inputdata = simplejson.loads(inputdata_str)

    if 'StepResult' in inputdata:
        if 'ActivisErr' in inputdata['StepResult']:
            return inputdata['StepResult']['result']['ActivisErr']
        
    return ''


import unittest,simplejson,sys
from InCli import InCli
from InCli.SFAPI import VlocityErrorLog,restClient,VlocityTrackingEntry

class Test_VlocityErrorEntry(unittest.TestCase):

    def test_get_errors(self):
        restClient.init('NOSPRD')

        VlocityErrorLog.get_errors()

        print()

    def test_get_errors(self):
        restClient.init('NOSPRD')

        VlocityErrorLog.get_errors()

        print()
    def test_print_errors_orderNum(self):
        restClient.init('NOSPRD')

        orderNums = [
            '00300871',
            '00300083',
            '00300052',
            '00298441',
            '00303789',
            '00303731',
            '00303549',
            '00303520',
            '00303458',
            '00303477',
            '00302987',
            '00303406',
            '00303322',
            '00302681',
            '00302623'
        ]

        original_stdout = sys.stdout
        _filename = f"vte_output.txt"
        with open(_filename, 'w') as f:
            sys.stdout = f 
            for orderNumber in orderNums:
                VlocityTrackingEntry.print_error_list(orderNumber=orderNumber)
                print()
            sys.stdout = original_stdout 
            print()
            print(f"   File {_filename} created.")

        print()
    def test_get_error_ip(self):
        restClient.init('NOSDEV')

        Id = 'a6K3O000000FHuqUAG'

        q = f"select fields(all) from vlocity_cmt__VlocityErrorLogEntry__c where Id='{Id}' limit 10"

        res = query.query(q)


        out=[]
        for record in res['records']:
            data = record['vlocity_cmt__InputData__c']
            datas = simplejson.loads(data)

            theFile =jsonFile.write('TheIPError_1',datas)
            
            out.append(datas)
            print(datas['ErrorMessage__c'])


      #  file_csv.write('VlocityErrorLogEntry_1',out)

        print()

        



import unittest,simplejson
from InCli import InCli
from InCli.SFAPI import file,ip_attachments,restClient,utils,file_csv,thread

class Test_Attachement(unittest.TestCase):
    def test_query(self):
        restClient.init('NOSQSM')
        rl = 2000
        res = ip_attachments.get_attachments(limit=rl)

        stats = []
        resp_out=[]

        stats_columns=[]

        def do_work(record):
            attachment = restClient.requestWithConnection(action=record['Body'])  
            attachment['BodyLength'] = record['BodyLength']
            return attachment   

        def on_done(attachment,stats):
            att_str = simplejson.dumps(attachment)

            out = self.obj_sizes(attachment)
            self.obj_size_percentage(out)

            resp = self.obj_sizes(attachment['response'])
            self.obj_size_percentage(resp)
            
            stat = {
                'BodyLength':attachment['BodyLength'],
                'JSONLenght':len(att_str),
                'name':attachment['lwcName'],
                'children%': int([ o['percentage'] for o in out if o['key']=='children'][0]),
                'childrenS': [ o['size'] for o in out if o['key']=='children'][0],
                'labelMap%': int([ o['percentage'] for o in out if o['key']=='labelMap'][0]),
                'labelMapS': [ o['size'] for o in out if o['key']=='labelMap'][0],
                'response%': int([ o['percentage'] for o in out if o['key']=='response'][0]),
                'responseS': [ o['size'] for o in out if o['key']=='response'][0]
            } 

            for re in resp:
                if re['percentage'] > 5:
                    stat[re['key']] = int(re['size'])

            stats.append(stat)

        thread.execute_threaded(res['records'],stats,do_work,on_done,threads=20)

        for stat in stats:
            for key in stat.keys():
                if key not in stats_columns: stats_columns.append(key)
        
        utils.printFormated(stats,fieldsString=":".join(stats_columns))

        file_csv.write(f'attachment_stats_size_{rl}',stats,header_columns_list=stats_columns)

        print()

    def obj_size_percentage(self,obj):
        sum = 0
        for o in obj:
            sum = sum + o['size']

        for o in obj:
            o['percentage'] = 100*o['size']/sum

    def obj_sizes(self,obj):
        out =[] 
        if type(obj) is list:
            for x,r in enumerate(obj):
                o = {'key':x,'size':self.getsize(r)}
                out.append(o)
            return out
                
        for key in obj.keys():
            val = obj[key]
            o = {'key':key,'size':self.getsize(val)}
            out.append(o)
        return out

    def getsize(self,value):
        if value == None: return 0
        if type(value) is int: return 1
        if type(value) is float: return 4

        if type(value) is str: return len(value)
        if type(value) is bool: return 1

        if type(value) is dict:
            val_str = simplejson.dumps(value)
            return len(val_str)
        
        if type(value) is list:
            val_str = simplejson.dumps(value)
            return len(val_str)          

        print(type(value))

        a=1
        



import unittest
from InCli import InCli
from InCli.SFAPI import file,restClient,tooling,Sobjects,utils,traceFlag,query

class Test_TraceFlag(unittest.TestCase):
    def test_set_InCli_for_user(self):
        restClient.init('NOSDEV')

     #   userId = Sobjects.IdF('User','username:uormaechea_nosdev@nos.pt')
        userF='username:pvg@optimus.force.com.nosdev'
        InCli_trace_flags = traceFlag.get_InCli_traceflags_for_user(userF)

        if InCli_trace_flags == None:
            InCli_trace_flags = traceFlag.create_trace_flag_incli_f(userF)

        InCli_trace_flags = traceFlag.get_InCli_traceflags_for_user(userF)

        InCli_trace_flags = traceFlag.update_trace_flag_incli(InCli_trace_flags['Id'])

        traceFlag.delete_trace_Flag(InCli_trace_flags['Id'])

        print()

    def test_create_debug_level(self):
        restClient.init('DTI')
        traceFlag.create_debug_level_incli()

    def test_set_incli_traceFlag_for_user(self):
        restClient.init('DTI')

        userF = "username:onboarding@nosdti-parceiros.cs109.force.com"
        traceFlag.set_incli_traceFlag_for_user(userF)

    def test_debuglevel_get(self):
        restClient.init('NOSDEV')

        call = tooling.tooling_IdF('DebugLevel',"DeveloperName:XXXX")   
        self.assertTrue(call==None)

        id = tooling.tooling_IdF('DebugLevel',"DeveloperName:InCli")   
        if id != None:
            tooling.tooling_delete('DebugLevel',id)
            id = tooling.tooling_IdF('DebugLevel',"DeveloperName:InCli")   

            print()
        if id == None:
            tooling.create_debug_level_incli()
            print()

    def test_query_who(self):

        envs = ['DTI',"NOSDEV","NOSQSM","NOSPRD"]
        for env in envs:
            try:
                restClient.init(env)

                DebugLevelId = tooling.IdF('DebugLevel',"DeveloperName:InCli")   
                q = f"select TracedEntityId,StartDate,ExpirationDate from TraceFlag where DebugLevelId='{DebugLevelId}' limit 100"

                res =tooling.query(q)
                q1 = f"select username from User where Id='{res['records'][0]['TracedEntityId']}'"
                res1 = query.query(q1)
                print()
                print(f'Environment: {env}')
                utils.printFormated(res1['records'])
            except Exception as e:
                utils.printException(e)

        print()

    def test_completions(self):
        restClient.init('NOSDEV')
        tooling.completions()

    def test_aync(self):
        restClient.init('NOSDEV')
        tooling.executeAnonymous()

    def test_limitUsageHistory(self):
        restClient.init('NOSDEV')

        q = "select fields(all) from NetworkPublicUsageDailyMetrics limit 10 "

        res = tooling.query(q)

        print()



from . import restClient,CPQ,account,timeStats

def create_cart_with_promotion(cartName,accountName,pricelistName,promoName,ts:timeStats=None):
#    restClient.setLoggingLevel()
#    cartName = 'testa_4'

    cartId = CPQ.getCartId(f"name:{cartName}")
    if cartId != None:
        CPQ.deleteCart(cartId)

    accountId = account.createAccount_Id(f'Name:{accountName}',recordTypeName='Consumer')
    cartId = CPQ.createCart(accountF= accountId, pricelistF=f'Name:{pricelistName}',name=cartName,checkExists=True)
    if ts!=None: ts.time('createCart')
    #print(restClient.getLastCallElapsedTime())

   # promoName = 'NOS4u 100Mb + Móvel'
    promo = CPQ.getCartPromotions(cartId,query=promoName,onlyOne=True)

    res = CPQ.postCartsPromoItems_api(cartId,promo['Id'])
    print(restClient.getLastCallElapsedTime())

    all = CPQ.getCartItems_api(cartId,fields='vlocity_cmt__ServiceAccountId__c,vlocity_cmt__BillingAccountId__c',price=False,validate=False,includeAttachment=False)
    print(restClient.getLastCallElapsedTime())